@@ -52,7 +52,7 @@ AgentMsgFilterResult agent_msg_filter_process_data(AgentMsgFilter *filter,
     struct VDAgentMessage msg_header;
 
     if (len > VD_AGENT_MAX_DATA_SIZE) {
-        spice_printerr("invalid agent message: too large");
+        g_warning("invalid agent message: too large");
         return AGENT_MSG_FILTER_PROTO_ERROR;
     }
 
@@ -60,7 +60,7 @@ AgentMsgFilterResult agent_msg_filter_process_data(AgentMsgFilter *filter,
     if (filter->msg_data_to_read) {
 data_to_read:
         if (len > filter->msg_data_to_read) {
-            spice_printerr("invalid agent message: data exceeds size from header");
+            g_warning("invalid agent message: data exceeds size from header");
             return AGENT_MSG_FILTER_PROTO_ERROR;
         }
         filter->msg_data_to_read -= len;
@@ -68,49 +68,45 @@ data_to_read:
     }
 
     if (len < sizeof(msg_header)) {
-        spice_printerr("invalid agent message: incomplete header");
+        g_warning("invalid agent message: incomplete header");
         return AGENT_MSG_FILTER_PROTO_ERROR;
     }
     memcpy(&msg_header, data, sizeof(msg_header));
     len -= sizeof(msg_header);
 
     if (msg_header.protocol != VD_AGENT_PROTOCOL) {
-        spice_printerr("invalid agent protocol: %u", msg_header.protocol);
+        g_warning("invalid agent protocol: %u", msg_header.protocol);
         return AGENT_MSG_FILTER_PROTO_ERROR;
     }
 
     if (filter->discard_all) {
         filter->result = AGENT_MSG_FILTER_DISCARD;
     } else {
+        // default, easier to set once
+        filter->result = AGENT_MSG_FILTER_OK;
         switch (msg_header.type) {
         case VD_AGENT_CLIPBOARD:
         case VD_AGENT_CLIPBOARD_GRAB:
         case VD_AGENT_CLIPBOARD_REQUEST:
         case VD_AGENT_CLIPBOARD_RELEASE:
-            if (filter->copy_paste_enabled) {
-                filter->result = AGENT_MSG_FILTER_OK;
-            } else {
+            if (!filter->copy_paste_enabled) {
                 filter->result = AGENT_MSG_FILTER_DISCARD;
             }
             break;
         case VD_AGENT_FILE_XFER_START:
         case VD_AGENT_FILE_XFER_STATUS:
         case VD_AGENT_FILE_XFER_DATA:
-            if (filter->file_xfer_enabled) {
-                filter->result = AGENT_MSG_FILTER_OK;
-            } else {
+            if (!filter->file_xfer_enabled) {
                 filter->result = AGENT_MSG_FILTER_DISCARD;
             }
             break;
         case VD_AGENT_MONITORS_CONFIG:
             if (filter->use_client_monitors_config) {
                 filter->result = AGENT_MSG_FILTER_MONITORS_CONFIG;
-            } else {
-                filter->result = AGENT_MSG_FILTER_OK;
             }
             break;
         default:
-            filter->result = AGENT_MSG_FILTER_OK;
+            break;
         }
     }
 
