@@ -78,6 +78,9 @@ struct RedCharDeviceClass
      * due to slow flow or due to some other error.
      * The called instance should disconnect the client, or at least the corresponding channel */
     void (*remove_client)(RedCharDevice *self, RedClient *client);
+
+    /* This cb is called when device receives an event */
+    void (*port_event)(RedCharDevice *self, uint8_t event);
 };
 
 GType red_char_device_get_type(void) G_GNUC_CONST;
@@ -215,10 +218,14 @@ void red_char_device_send_to_client_tokens_set(RedCharDevice *dev,
                                                uint32_t tokens);
 /** Write to device **/
 
-RedCharDeviceWriteBuffer *red_char_device_write_buffer_get(RedCharDevice *dev,
-                                                           RedClient *client, int size);
-RedCharDeviceWriteBuffer *red_char_device_write_buffer_get_server_no_token(
-    RedCharDevice *dev, int size);
+RedCharDeviceWriteBuffer *red_char_device_write_buffer_get_client(RedCharDevice *dev,
+                                                                  RedClient *client,
+                                                                  int size);
+
+/* Returns NULL if use_token == true and no tokens are available */
+RedCharDeviceWriteBuffer *red_char_device_write_buffer_get_server(RedCharDevice *dev,
+                                                                  int size,
+                                                                  bool use_token);
 
 /* Either add the buffer to the write queue or release it */
 void red_char_device_write_buffer_add(RedCharDevice *dev,
