@@ -18,7 +18,6 @@
 #include <config.h>
 #endif
 
-#include <arpa/inet.h>
 #include <lz4.h>
 #include "red-common.h"
 #include "lz4-encoder.h"
@@ -74,7 +73,7 @@ int lz4_encode(Lz4EncoderContext *lz4, int height, int stride, uint8_t *io_ptr,
         in_size = stride * num_lines;
         lines += in_size;
         int bound_size = LZ4_compressBound(in_size);
-        compressed_lines = (uint8_t *) malloc(bound_size + 4);
+        compressed_lines = g_new(uint8_t, bound_size + 4);
 #ifdef HAVE_LZ4_COMPRESS_FAST_CONTINUE
         enc_size = LZ4_compress_fast_continue(stream, (const char *) in_buf,
                                               (char *) compressed_lines + 4, in_size,
@@ -85,11 +84,12 @@ int lz4_encode(Lz4EncoderContext *lz4, int height, int stride, uint8_t *io_ptr,
 #endif
         if (enc_size <= 0) {
             spice_error("compress failed!");
-            free(compressed_lines);
+            g_free(compressed_lines);
             LZ4_freeStream(stream);
             return 0;
         }
-        *((uint32_t *)compressed_lines) = htonl(enc_size);
+        // compressed_lines is returned by malloc so is surely aligned
+        *SPICE_ALIGNED_CAST(uint32_t *, compressed_lines) = GUINT32_TO_BE(enc_size);
 
         out_size += enc_size += 4;
         already_copied = 0;
@@ -100,7 +100,7 @@ int lz4_encode(Lz4EncoderContext *lz4, int height, int stride, uint8_t *io_ptr,
             num_io_bytes = enc->usr->more_space(enc->usr, &io_ptr);
             if (num_io_bytes <= 0) {
                 spice_error("more space failed");
-                free(compressed_lines);
+                g_free(compressed_lines);
                 LZ4_freeStream(stream);
                 return 0;
             }
@@ -110,13 +110,13 @@ int lz4_encode(Lz4EncoderContext *lz4, int height, int stride, uint8_t *io_ptr,
         out_buf += enc_size;
         num_io_bytes -= enc_size;
 
-        free(compressed_lines);
+        g_free(compressed_lines);
         total_lines += num_lines;
     } while (total_lines < height);
 
     LZ4_freeStream(stream);
     if (total_lines != height) {
-        spice_error("too many lines\n");
+        spice_error("too many lines");
         out_size = 0;
     }
 
