@@ -15,8 +15,8 @@
    License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef _H_MIGRATION_PROTOCOL
-#define _H_MIGRATION_PROTOCOL
+#ifndef MIGRATION_PROTOCOL_H_
+#define MIGRATION_PROTOCOL_H_
 
 #include <spice/macros.h>
 #include <spice/vd_agent.h>
@@ -212,4 +212,4 @@ static inline int migration_protocol_validate_header(SpiceMigrateDataHeader *hea
     return TRUE;
 }
 
-#endif
+#endif /* MIGRATION_PROTOCOL_H_ */
