@@ -191,18 +191,18 @@ static uint32_t get_min_required_playback_delay(uint64_t frame_enc_size,
 static void mjpeg_video_buffer_free(VideoBuffer *video_buffer)
 {
     MJpegVideoBuffer *buffer = (MJpegVideoBuffer*)video_buffer;
-    free(buffer->base.data);
-    free(buffer);
+    g_free(buffer->base.data);
+    g_free(buffer);
 }
 
 static MJpegVideoBuffer* create_mjpeg_video_buffer(void)
 {
-    MJpegVideoBuffer *buffer = spice_new0(MJpegVideoBuffer, 1);
+    MJpegVideoBuffer *buffer = g_new0(MJpegVideoBuffer, 1);
     buffer->base.free = mjpeg_video_buffer_free;
     buffer->maxsize = MJPEG_INITIAL_BUFFER_SIZE;
-    buffer->base.data = malloc(buffer->maxsize);
+    buffer->base.data = g_try_malloc(buffer->maxsize);
     if (!buffer->base.data) {
-        free(buffer);
+        g_free(buffer);
         buffer = NULL;
     }
     return buffer;
@@ -211,10 +211,10 @@ static MJpegVideoBuffer* create_mjpeg_video_buffer(void)
 static void mjpeg_encoder_destroy(VideoEncoder *video_encoder)
 {
     MJpegEncoder *encoder = (MJpegEncoder*)video_encoder;
-    free(encoder->cinfo.dest);
+    g_free(encoder->cinfo.dest);
     jpeg_destroy_compress(&encoder->cinfo);
-    free(encoder->row);
-    free(encoder);
+    g_free(encoder->row);
+    g_free(encoder);
 }
 
 static uint8_t mjpeg_encoder_get_bytes_per_pixel(MJpegEncoder *encoder)
@@ -278,7 +278,7 @@ static boolean empty_mem_output_buffer(j_compress_ptr cinfo)
 
   /* Try to allocate new buffer with double size */
   nextsize = dest->bufsize * 2;
-  nextbuffer = realloc(dest->buffer, nextsize);
+  nextbuffer = g_try_realloc(dest->buffer, nextsize);
 
   if (nextbuffer == NULL)
     ERREXIT1(cinfo, JERR_OUT_OF_MEMORY, 10);
@@ -304,7 +304,7 @@ static void term_mem_destination(j_compress_ptr cinfo)
  * Prepare for output to a memory buffer.
  * The caller must supply its own initial buffer and size.
  * When the actual data output exceeds the given size, the library
- * will adapt the buffer size as necessary using the malloc()/free()
+ * will adapt the buffer size as necessary using the g_malloc()/g_free()
  * functions. The buffer is available to the application after the
  * compression and the application is then responsible for freeing it.
  */
@@ -323,7 +323,7 @@ spice_jpeg_mem_dest(j_compress_ptr cinfo,
    * can be written to the same buffer without re-executing jpeg_mem_dest.
    */
   if (cinfo->dest == NULL) { /* first time for this JPEG object? */
-    cinfo->dest = spice_malloc(sizeof(mem_destination_mgr));
+    cinfo->dest = g_malloc(sizeof(mem_destination_mgr));
   }
 
   dest = (mem_destination_mgr *) cinfo->dest;
@@ -614,7 +614,8 @@ static void mjpeg_encoder_adjust_params_to_bit_rate(MJpegEncoder *encoder)
 
     spice_debug("cur-fps=%u new-fps=%u (new/old=%.2f) |"
                 "bit-rate=%.2f (Mbps) latency=%u (ms) quality=%d |"
-                " new-size-avg %"PRIu64" , base-size %"PRIu64", (new/old=%.2f) ",
+                " new-size-avg %"G_GUINT64_FORMAT" ,"
+                " base-size %"G_GUINT64_FORMAT", (new/old=%.2f) ",
                 rate_control->fps, new_fps, ((double)new_fps)/rate_control->fps,
                 ((double)rate_control->byte_rate*8)/1024/1024,
                 latency,
@@ -676,7 +677,8 @@ static void mjpeg_encoder_adjust_fps(MJpegEncoder *encoder, uint64_t now)
 
         avg_fps = ((double)rate_control->adjusted_fps_num_frames * MSEC_PER_SEC) /
                   adjusted_fps_time_passed;
-        spice_debug("#frames-adjust=%"PRIu64" #adjust-time=%"PRIu64" avg-fps=%.2f",
+        spice_debug("#frames-adjust=%"G_GUINT64_FORMAT
+                    " #adjust-time=%"G_GUINT64_FORMAT" avg-fps=%.2f",
                     rate_control->adjusted_fps_num_frames, adjusted_fps_time_passed, avg_fps);
         spice_debug("defined=%u old-adjusted=%.2f", rate_control->fps, rate_control->adjusted_fps);
         fps_ratio = avg_fps / rate_control->fps;
@@ -700,7 +702,7 @@ static void mjpeg_encoder_adjust_fps(MJpegEncoder *encoder, uint64_t now)
 }
 
 /*
- * dest must be either NULL or allocated by malloc, since it might be freed
+ * dest must be either NULL or allocated by g_malloc, since it might be freed
  * during the encoding, if its size is too small.
  *
  * return:
@@ -790,7 +792,7 @@ static int mjpeg_encoder_start_frame(MJpegEncoder *encoder,
             return VIDEO_ENCODER_FRAME_UNSUPPORTED;
         }
         if (encoder->row_size < stride) {
-            encoder->row = spice_realloc(encoder->row, stride);
+            encoder->row = g_realloc(encoder->row, stride);
             encoder->row_size = stride;
         }
     }
@@ -983,7 +985,7 @@ static void mjpeg_encoder_quality_eval_stop(MJpegEncoder *encoder)
         fps = MJPEG_MAX_FPS / 2;
         break;
     default:
-        spice_warning("unexected");
+        spice_warning("unexpected");
         return;
     }
     mjpeg_encoder_reset_quality(encoder, quality_id, fps, 0);
@@ -1025,7 +1027,7 @@ static void mjpeg_encoder_decrease_bit_rate(MJpegEncoder *encoder)
         measured_byte_rate = bit_rate_info->sum_enc_size / duration_sec;
         measured_fps = bit_rate_info->num_enc_frames / duration_sec;
         decrease_size = bit_rate_info->sum_enc_size / bit_rate_info->num_enc_frames;
-        spice_debug("bit rate esitimation %.2f (Mbps) fps %u",
+        spice_debug("bit rate estimation %.2f (Mbps) fps %u",
                     measured_byte_rate*8/1024.0/1024,
                     measured_fps);
     } else {
@@ -1094,7 +1096,7 @@ static void mjpeg_encoder_increase_bit_rate(MJpegEncoder *encoder)
         measured_byte_rate = bit_rate_info->sum_enc_size / duration_sec;
         measured_fps = bit_rate_info->num_enc_frames / duration_sec;
         avg_frame_size = bit_rate_info->sum_enc_size / bit_rate_info->num_enc_frames;
-        spice_debug("bit rate esitimation %.2f (Mbps) defined %.2f"
+        spice_debug("bit rate estimation %.2f (Mbps) defined %.2f"
                     " fps %u avg-frame-size=%.2f (KB)",
                     measured_byte_rate*8/1024.0/1024,
                     rate_control->byte_rate*8/1024.0/1024,
@@ -1357,7 +1359,7 @@ VideoEncoder *mjpeg_encoder_new(SpiceVideoCodecType codec_type,
 
     spice_return_val_if_fail(codec_type == SPICE_VIDEO_CODEC_TYPE_MJPEG, NULL);
 
-    encoder = spice_new0(MJpegEncoder, 1);
+    encoder = g_new0(MJpegEncoder, 1);
     encoder->base.destroy = mjpeg_encoder_destroy;
     encoder->base.encode_frame = mjpeg_encoder_encode_frame;
     encoder->base.client_stream_report = mjpeg_encoder_client_stream_report;
