@@ -24,7 +24,6 @@
 #include <errno.h>
 #include <string.h>
 #include <pthread.h>
-#include <sys/socket.h>
 #include <inttypes.h>
 
 #include <spice/qxl_dev.h>
@@ -37,14 +36,12 @@
 #include "dispatcher.h"
 #include "red-parse-qxl.h"
 #include "red-channel-client.h"
+#include "display-limits.h"
 
 #include "red-qxl.h"
 
 
-struct AsyncCommand {
-    RedWorkerMessage message;
-    uint64_t cookie;
-};
+#define MAX_MONITORS_COUNT 16
 
 struct QXLState {
     QXLWorker qxl_worker;
@@ -55,16 +52,34 @@ struct QXLState {
     int x_res;
     int y_res;
     int use_hardware_cursor;
-    QXLDevSurfaceCreate surface_create;
     unsigned int max_monitors;
     RedsState *reds;
     RedWorker *worker;
+    char device_address[MAX_DEVICE_ADDRESS_LEN];
+    uint32_t device_display_ids[MAX_MONITORS_COUNT];
+    size_t monitors_count;  // length of ^^^
+
+    bool running;
 
     pthread_mutex_t scanout_mutex;
     SpiceMsgDisplayGlScanoutUnix scanout;
-    struct AsyncCommand *gl_draw_async;
+    uint64_t gl_draw_cookie;
 };
 
+#define GL_DRAW_COOKIE_INVALID (~((uint64_t) 0))
+
+/* used by RedWorker */
+bool red_qxl_is_running(QXLInstance *qxl)
+{
+    return qxl->st->running;
+}
+
+/* used by RedWorker */
+void red_qxl_set_running(QXLInstance *qxl, bool running)
+{
+    qxl->st->running = running;
+}
+
 int red_qxl_check_qxl_version(QXLInstance *qxl, int major, int minor)
 {
     int qxl_major = qxl_get_interface(qxl)->base.major_version;
@@ -74,121 +89,6 @@ int red_qxl_check_qxl_version(QXLInstance *qxl, int major, int minor)
             ((qxl_major == major) && (qxl_minor >= minor)));
 }
 
-static void red_qxl_set_display_peer(RedChannel *channel, RedClient *client,
-                                     RedsStream *stream, int migration,
-                                     RedChannelCapabilities *caps)
-{
-    RedWorkerMessageDisplayConnect payload = {0,};
-    Dispatcher *dispatcher;
-
-    spice_debug("%s", "");
-    dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-    payload.client = client;
-    payload.stream = stream;
-    payload.migration = migration;
-    red_channel_capabilities_init(&payload.caps, caps);
-
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_DISPLAY_CONNECT,
-                            &payload);
-}
-
-static void red_qxl_disconnect_display_peer(RedChannelClient *rcc)
-{
-    RedWorkerMessageDisplayDisconnect payload;
-    Dispatcher *dispatcher;
-    RedChannel *channel = red_channel_client_get_channel(rcc);
-
-    if (!channel) {
-        return;
-    }
-
-    dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-
-    spice_printerr("");
-    payload.rcc = rcc;
-
-    // TODO: we turned it to be sync, due to client_destroy . Should we support async? - for this we will need ref count
-    // for channels
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_DISPLAY_DISCONNECT,
-                            &payload);
-}
-
-static void red_qxl_display_migrate(RedChannelClient *rcc)
-{
-    RedWorkerMessageDisplayMigrate payload;
-    Dispatcher *dispatcher;
-    RedChannel *channel = red_channel_client_get_channel(rcc);
-    uint32_t type, id;
-
-    if (!channel) {
-        return;
-    }
-    g_object_get(channel, "channel-type", &type, "id", &id, NULL);
-    dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-    spice_printerr("channel type %u id %u", type, id);
-    payload.rcc = rcc;
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_DISPLAY_MIGRATE,
-                            &payload);
-}
-
-static void red_qxl_set_cursor_peer(RedChannel *channel, RedClient *client, RedsStream *stream,
-                                    int migration,
-                                    RedChannelCapabilities *caps)
-{
-    RedWorkerMessageCursorConnect payload = {0,};
-    Dispatcher *dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-    spice_printerr("");
-    payload.client = client;
-    payload.stream = stream;
-    payload.migration = migration;
-    red_channel_capabilities_init(&payload.caps, caps);
-
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_CURSOR_CONNECT,
-                            &payload);
-}
-
-static void red_qxl_disconnect_cursor_peer(RedChannelClient *rcc)
-{
-    RedWorkerMessageCursorDisconnect payload;
-    Dispatcher *dispatcher;
-    RedChannel *channel = red_channel_client_get_channel(rcc);
-
-    if (!channel) {
-        return;
-    }
-
-    dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-    spice_printerr("");
-    payload.rcc = rcc;
-
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_CURSOR_DISCONNECT,
-                            &payload);
-}
-
-static void red_qxl_cursor_migrate(RedChannelClient *rcc)
-{
-    RedWorkerMessageCursorMigrate payload;
-    Dispatcher *dispatcher;
-    RedChannel *channel = red_channel_client_get_channel(rcc);
-    uint32_t type, id;
-
-    if (!channel) {
-        return;
-    }
-    g_object_get(channel, "channel-type", &type, "id", &id, NULL);
-    dispatcher = (Dispatcher *)g_object_get_data(G_OBJECT(channel), "dispatcher");
-    spice_printerr("channel type %u id %u", type, id);
-    payload.rcc = rcc;
-    dispatcher_send_message(dispatcher,
-                            RED_WORKER_MESSAGE_CURSOR_MIGRATE,
-                            &payload);
-}
-
 static void red_qxl_update_area(QXLState *qxl_state, uint32_t surface_id,
                                 QXLRect *qxl_area, QXLRect *qxl_dirty_rects,
                                 uint32_t num_dirty_rects, uint32_t clear_dirty_region)
@@ -205,33 +105,14 @@ static void red_qxl_update_area(QXLState *qxl_state, uint32_t surface_id,
                             &payload);
 }
 
-gboolean red_qxl_use_client_monitors_config(QXLInstance *qxl)
-{
-    return (red_qxl_check_qxl_version(qxl, 3, 3) &&
-        qxl_get_interface(qxl)->client_monitors_config &&
-        qxl_get_interface(qxl)->client_monitors_config(qxl, NULL));
-}
-
 gboolean red_qxl_client_monitors_config(QXLInstance *qxl,
                                         VDAgentMonitorsConfig *monitors_config)
 {
-    return (qxl_get_interface(qxl)->client_monitors_config &&
+    return (red_qxl_check_qxl_version(qxl, 3, 3) &&
+        qxl_get_interface(qxl)->client_monitors_config &&
         qxl_get_interface(qxl)->client_monitors_config(qxl, monitors_config));
 }
 
-static AsyncCommand *async_command_alloc(QXLState *qxl_state,
-                                         RedWorkerMessage message,
-                                         uint64_t cookie)
-{
-    AsyncCommand *async_command = spice_new0(AsyncCommand, 1);
-
-    async_command->cookie = cookie;
-    async_command->message = message;
-
-    spice_debug("%p", async_command);
-    return async_command;
-}
-
 static void red_qxl_update_area_async(QXLState *qxl_state,
                                       uint32_t surface_id,
                                       QXLRect *qxl_area,
@@ -241,7 +122,7 @@ static void red_qxl_update_area_async(QXLState *qxl_state,
     RedWorkerMessage message = RED_WORKER_MESSAGE_UPDATE_ASYNC;
     RedWorkerMessageUpdateAsync payload;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.surface_id = surface_id;
     payload.qxl_area = *qxl_area;
     payload.clear_dirty_region = clear_dirty_region;
@@ -280,7 +161,7 @@ static void red_qxl_add_memslot_async(QXLState *qxl_state, QXLDevMemSlot *mem_sl
     RedWorkerMessageAddMemslotAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_ADD_MEMSLOT_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.mem_slot = *mem_slot;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
 }
@@ -321,11 +202,12 @@ static void red_qxl_destroy_surfaces_async(QXLState *qxl_state, uint64_t cookie)
     RedWorkerMessageDestroySurfacesAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_DESTROY_SURFACES_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
 }
 
-static void red_qxl_destroy_primary_surface_complete(QXLState *qxl_state)
+/* used by RedWorker */
+void red_qxl_destroy_primary_surface_complete(QXLState *qxl_state)
 {
     qxl_state->x_res = 0;
     qxl_state->y_res = 0;
@@ -354,7 +236,7 @@ red_qxl_destroy_primary_surface_async(QXLState *qxl_state,
     RedWorkerMessageDestroyPrimarySurfaceAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_DESTROY_PRIMARY_SURFACE_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.surface_id = surface_id;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
 }
@@ -376,17 +258,16 @@ static void qxl_worker_destroy_primary_surface(QXLWorker *qxl_worker, uint32_t s
     red_qxl_destroy_primary_surface(qxl_state, surface_id, 0, 0);
 }
 
-static void red_qxl_create_primary_surface_complete(QXLState *qxl_state)
+/* used by RedWorker */
+void red_qxl_create_primary_surface_complete(QXLState *qxl_state, const QXLDevSurfaceCreate *surface)
 {
-    QXLDevSurfaceCreate *surface = &qxl_state->surface_create;
-
     qxl_state->x_res = surface->width;
     qxl_state->y_res = surface->height;
-    qxl_state->use_hardware_cursor = surface->mouse_mode;
+    // mouse_mode is a boolean value, enforce it
+    qxl_state->use_hardware_cursor = !!surface->mouse_mode;
     qxl_state->primary_active = TRUE;
 
     reds_update_client_mouse_allowed(qxl_state->reds);
-    memset(&qxl_state->surface_create, 0, sizeof(QXLDevSurfaceCreate));
 }
 
 static void
@@ -396,8 +277,7 @@ red_qxl_create_primary_surface_async(QXLState *qxl_state, uint32_t surface_id,
     RedWorkerMessageCreatePrimarySurfaceAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_CREATE_PRIMARY_SURFACE_ASYNC;
 
-    qxl_state->surface_create = *surface;
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.surface_id = surface_id;
     payload.surface = *surface;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
@@ -409,31 +289,19 @@ red_qxl_create_primary_surface_sync(QXLState *qxl_state, uint32_t surface_id,
 {
     RedWorkerMessageCreatePrimarySurface payload = {0,};
 
-    qxl_state->surface_create = *surface;
     payload.surface_id = surface_id;
     payload.surface = *surface;
     dispatcher_send_message(qxl_state->dispatcher,
                             RED_WORKER_MESSAGE_CREATE_PRIMARY_SURFACE,
                             &payload);
-    red_qxl_create_primary_surface_complete(qxl_state);
-}
-
-static void
-red_qxl_create_primary_surface(QXLState *qxl_state, uint32_t surface_id,
-                               QXLDevSurfaceCreate *surface, int async, uint64_t cookie)
-{
-    if (async) {
-        red_qxl_create_primary_surface_async(qxl_state, surface_id, surface, cookie);
-    } else {
-        red_qxl_create_primary_surface_sync(qxl_state, surface_id, surface);
-    }
+    red_qxl_create_primary_surface_complete(qxl_state, surface);
 }
 
 static void qxl_worker_create_primary_surface(QXLWorker *qxl_worker, uint32_t surface_id,
                                       QXLDevSurfaceCreate *surface)
 {
     QXLState *qxl_state = SPICE_CONTAINEROF(qxl_worker, QXLState, qxl_worker);
-    red_qxl_create_primary_surface(qxl_state, surface_id, surface, 0, 0);
+    red_qxl_create_primary_surface_sync(qxl_state, surface_id, surface);
 }
 
 static void red_qxl_reset_image_cache(QXLState *qxl_state)
@@ -484,26 +352,15 @@ static void red_qxl_destroy_surface_wait_async(QXLState *qxl_state,
     RedWorkerMessageDestroySurfaceWaitAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_DESTROY_SURFACE_WAIT_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.surface_id = surface_id;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
 }
 
-static void red_qxl_destroy_surface_wait(QXLState *qxl_state,
-                                         uint32_t surface_id,
-                                         int async, uint64_t cookie)
-{
-    if (async) {
-        red_qxl_destroy_surface_wait_async(qxl_state, surface_id, cookie);
-    } else {
-        red_qxl_destroy_surface_wait_sync(qxl_state, surface_id);
-    }
-}
-
 static void qxl_worker_destroy_surface_wait(QXLWorker *qxl_worker, uint32_t surface_id)
 {
     QXLState *qxl_state = SPICE_CONTAINEROF(qxl_worker, QXLState, qxl_worker);
-    red_qxl_destroy_surface_wait(qxl_state, surface_id, 0, 0);
+    red_qxl_destroy_surface_wait_sync(qxl_state, surface_id);
 }
 
 static void red_qxl_reset_memslots(QXLState *qxl_state)
@@ -588,7 +445,7 @@ static void red_qxl_flush_surfaces_async(QXLState *qxl_state, uint64_t cookie)
     RedWorkerMessageFlushSurfacesAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_FLUSH_SURFACES_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     dispatcher_send_message(qxl_state->dispatcher, message, &payload);
 }
 
@@ -600,7 +457,7 @@ static void red_qxl_monitors_config_async(QXLState *qxl_state,
     RedWorkerMessageMonitorsConfigAsync payload;
     RedWorkerMessage message = RED_WORKER_MESSAGE_MONITORS_CONFIG_ASYNC;
 
-    payload.base.cmd = async_command_alloc(qxl_state, message, cookie);
+    payload.base.cookie = cookie;
     payload.monitors_config = monitors_config;
     payload.group_id = group_id;
     payload.max_monitors = qxl_state->max_monitors;
@@ -638,7 +495,6 @@ static void red_qxl_loadvm_commands(QXLState *qxl_state,
 {
     RedWorkerMessageLoadvmCommands payload;
 
-    spice_printerr("");
     payload.count = count;
     payload.ext = ext;
     dispatcher_send_message(qxl_state->dispatcher,
@@ -654,18 +510,6 @@ static void qxl_worker_loadvm_commands(QXLWorker *qxl_worker,
     red_qxl_loadvm_commands(qxl_state, ext, count);
 }
 
-void red_qxl_attach_worker(QXLInstance *qxl)
-{
-    QXLInterface *interface = qxl_get_interface(qxl);
-    interface->attache_worker(qxl, &qxl->st->qxl_worker);
-}
-
-void red_qxl_set_compression_level(QXLInstance *qxl, int level)
-{
-    QXLInterface *interface = qxl_get_interface(qxl);
-    interface->set_compression_level(qxl, level);
-}
-
 uint32_t red_qxl_get_ram_size(QXLInstance *qxl)
 {
     QXLDevInitInfo qxl_info;
@@ -742,7 +586,7 @@ SPICE_GNUC_VISIBLE
 void spice_qxl_create_primary_surface(QXLInstance *instance, uint32_t surface_id,
                                 QXLDevSurfaceCreate *surface)
 {
-    red_qxl_create_primary_surface(instance->st, surface_id, surface, 0, 0);
+    red_qxl_create_primary_surface_sync(instance->st, surface_id, surface);
 }
 
 SPICE_GNUC_VISIBLE
@@ -760,7 +604,7 @@ void spice_qxl_reset_cursor(QXLInstance *instance)
 SPICE_GNUC_VISIBLE
 void spice_qxl_destroy_surface_wait(QXLInstance *instance, uint32_t surface_id)
 {
-    red_qxl_destroy_surface_wait(instance->st, surface_id, 0, 0);
+    red_qxl_destroy_surface_wait_sync(instance->st, surface_id);
 }
 
 SPICE_GNUC_VISIBLE
@@ -799,13 +643,13 @@ SPICE_GNUC_VISIBLE
 void spice_qxl_create_primary_surface_async(QXLInstance *instance, uint32_t surface_id,
                                 QXLDevSurfaceCreate *surface, uint64_t cookie)
 {
-    red_qxl_create_primary_surface(instance->st, surface_id, surface, 1, cookie);
+    red_qxl_create_primary_surface_async(instance->st, surface_id, surface, cookie);
 }
 
 SPICE_GNUC_VISIBLE
 void spice_qxl_destroy_surface_async(QXLInstance *instance, uint32_t surface_id, uint64_t cookie)
 {
-    red_qxl_destroy_surface_wait(instance->st, surface_id, 1, cookie);
+    red_qxl_destroy_surface_wait_async(instance->st, surface_id, cookie);
 }
 
 SPICE_GNUC_VISIBLE
@@ -861,11 +705,11 @@ void spice_qxl_gl_scanout(QXLInstance *qxl,
     spice_return_if_fail(qxl != NULL);
 
     QXLState *qxl_state = qxl->st;
-    spice_return_if_fail(qxl_state->gl_draw_async == NULL);
+    spice_return_if_fail(qxl_state->gl_draw_cookie == GL_DRAW_COOKIE_INVALID);
 
     pthread_mutex_lock(&qxl_state->scanout_mutex);
 
-    if (qxl_state->scanout.drm_dma_buf_fd != -1) {
+    if (qxl_state->scanout.drm_dma_buf_fd >= 0) {
         close(qxl_state->scanout.drm_dma_buf_fd);
     }
 
@@ -883,6 +727,8 @@ void spice_qxl_gl_scanout(QXLInstance *qxl,
     /* FIXME: find a way to coallesce all pending SCANOUTs */
     dispatcher_send_message(qxl_state->dispatcher,
                             RED_WORKER_MESSAGE_GL_SCANOUT, &payload);
+
+    reds_update_client_mouse_allowed(qxl_state->reds);
 }
 
 SPICE_GNUC_VISIBLE
@@ -904,66 +750,98 @@ void spice_qxl_gl_draw_async(QXLInstance *qxl,
 
     spice_return_if_fail(qxl != NULL);
     qxl_state = qxl->st;
-    if (qxl_state->scanout.drm_dma_buf_fd == -1) {
+    if (qxl_state->scanout.drm_dma_buf_fd < 0) {
         spice_warning("called spice_qxl_gl_draw_async without a buffer");
-        red_qxl_async_complete(qxl, async_command_alloc(qxl_state, message, cookie));
+        red_qxl_async_complete(qxl, cookie);
         return;
     }
-    spice_return_if_fail(qxl_state->gl_draw_async == NULL);
+    spice_return_if_fail(qxl_state->gl_draw_cookie == GL_DRAW_COOKIE_INVALID);
 
-    qxl_state->gl_draw_async = async_command_alloc(qxl_state, message, cookie);
+    qxl_state->gl_draw_cookie = cookie;
     dispatcher_send_message(qxl_state->dispatcher, message, &draw);
 }
 
-void red_qxl_async_complete(QXLInstance *qxl, AsyncCommand *async_command)
-{
-    QXLInterface *interface = qxl_get_interface(qxl);
-
-    spice_debug("%p: cookie %" PRId64, async_command, async_command->cookie);
-    switch (async_command->message) {
-    case RED_WORKER_MESSAGE_UPDATE_ASYNC:
-    case RED_WORKER_MESSAGE_ADD_MEMSLOT_ASYNC:
-    case RED_WORKER_MESSAGE_DESTROY_SURFACES_ASYNC:
-    case RED_WORKER_MESSAGE_DESTROY_SURFACE_WAIT_ASYNC:
-    case RED_WORKER_MESSAGE_FLUSH_SURFACES_ASYNC:
-    case RED_WORKER_MESSAGE_MONITORS_CONFIG_ASYNC:
-    case RED_WORKER_MESSAGE_GL_DRAW_ASYNC:
-        break;
-    case RED_WORKER_MESSAGE_CREATE_PRIMARY_SURFACE_ASYNC:
-        red_qxl_create_primary_surface_complete(qxl->st);
-        break;
-    case RED_WORKER_MESSAGE_DESTROY_PRIMARY_SURFACE_ASYNC:
-        red_qxl_destroy_primary_surface_complete(qxl->st);
-        break;
-    default:
-        spice_warning("unexpected message %d", async_command->message);
-    }
-    interface->async_complete(qxl, async_command->cookie);
-    free(async_command);
-}
-
 void red_qxl_gl_draw_async_complete(QXLInstance *qxl)
 {
     /* this reset before usage prevent a possible race condition */
-    struct AsyncCommand *async = qxl->st->gl_draw_async;
-    qxl->st->gl_draw_async = NULL;
-    red_qxl_async_complete(qxl, async);
+    uint64_t cookie = qxl->st->gl_draw_cookie;
+    qxl->st->gl_draw_cookie = GL_DRAW_COOKIE_INVALID;
+    red_qxl_async_complete(qxl, cookie);
+}
+
+SPICE_GNUC_VISIBLE
+void spice_qxl_set_device_info(QXLInstance *instance,
+                               const char *device_address,
+                               uint32_t device_display_id_start,
+                               uint32_t device_display_id_count)
+{
+    g_return_if_fail(device_address != NULL);
+
+    size_t da_len = strnlen(device_address, MAX_DEVICE_ADDRESS_LEN);
+    if (da_len >= MAX_DEVICE_ADDRESS_LEN) {
+        spice_error("Device address too long: %"G_GSIZE_FORMAT" > %u",
+                    da_len, MAX_DEVICE_ADDRESS_LEN);
+        return;
+    }
+
+    if (device_display_id_count > MAX_MONITORS_COUNT) {
+        spice_error("Device display ID count (%u) is greater than limit %u",
+                    device_display_id_count,
+                    MAX_MONITORS_COUNT);
+        return;
+    }
+
+    g_strlcpy(instance->st->device_address, device_address, MAX_DEVICE_ADDRESS_LEN);
+
+    g_debug("QXL Instance %d setting device address \"%s\" and monitor -> device display mapping:",
+            instance->id,
+            device_address);
+
+    // store the mapping monitor_id -> device_display_id
+    for (uint32_t monitor_id = 0; monitor_id < device_display_id_count; ++monitor_id) {
+        uint32_t device_display_id = device_display_id_start + monitor_id;
+        instance->st->device_display_ids[monitor_id] = device_display_id;
+        g_debug("   monitor ID %u -> device display ID %u",
+                monitor_id, device_display_id);
+    }
+
+    instance->st->monitors_count = device_display_id_count;
+    instance->st->max_monitors = device_display_id_count;
+
+    reds_send_device_display_info(red_qxl_get_server(instance->st));
+}
+
+const char* red_qxl_get_device_address(const QXLInstance *qxl)
+{
+    const QXLState *qxl_state = qxl->st;
+    return qxl_state->device_address;
+}
+
+const uint32_t* red_qxl_get_device_display_ids(const QXLInstance *qxl)
+{
+    const QXLState *qxl_state = qxl->st;
+    return qxl_state->device_display_ids;
+}
+
+size_t red_qxl_get_monitors_count(const QXLInstance *qxl)
+{
+    const QXLState *qxl_state = qxl->st;
+    return qxl_state->monitors_count;
 }
 
 void red_qxl_init(RedsState *reds, QXLInstance *qxl)
 {
     QXLState *qxl_state;
-    ClientCbs client_cursor_cbs = { NULL, };
-    ClientCbs client_display_cbs = { NULL, };
 
     spice_return_if_fail(qxl != NULL);
 
-    qxl_state = spice_new0(QXLState, 1);
+    qxl_state = g_new0(QXLState, 1);
     qxl_state->reds = reds;
     qxl_state->qxl = qxl;
     pthread_mutex_init(&qxl_state->scanout_mutex, NULL);
     qxl_state->scanout.drm_dma_buf_fd = -1;
-    qxl_state->dispatcher = dispatcher_new(RED_WORKER_MESSAGE_COUNT, NULL);
+    qxl_state->gl_draw_cookie = GL_DRAW_COOKIE_INVALID;
+    qxl_state->dispatcher = dispatcher_new(RED_WORKER_MESSAGE_COUNT);
     qxl_state->qxl_worker.major_version = SPICE_INTERFACE_QXL_MAJOR;
     qxl_state->qxl_worker.minor_version = SPICE_INTERFACE_QXL_MINOR;
     qxl_state->qxl_worker.wakeup = qxl_worker_wakeup;
@@ -986,17 +864,7 @@ void red_qxl_init(RedsState *reds, QXLInstance *qxl)
     qxl_state->max_monitors = UINT_MAX;
     qxl->st = qxl_state;
 
-    // TODO: move to their respective channel files
-    client_cursor_cbs.connect = red_qxl_set_cursor_peer;
-    client_cursor_cbs.disconnect = red_qxl_disconnect_cursor_peer;
-    client_cursor_cbs.migrate = red_qxl_cursor_migrate;
-
-    client_display_cbs.connect = red_qxl_set_display_peer;
-    client_display_cbs.disconnect = red_qxl_disconnect_display_peer;
-    client_display_cbs.migrate = red_qxl_display_migrate;
-
-    qxl_state->worker = red_worker_new(qxl, &client_cursor_cbs,
-                                       &client_display_cbs);
+    qxl_state->worker = red_worker_new(qxl);
 
     red_worker_run(qxl_state->worker);
 }
@@ -1017,7 +885,7 @@ void red_qxl_destroy(QXLInstance *qxl)
     /* this must be done after calling red_worker_free */
     qxl->st = NULL;
     pthread_mutex_destroy(&qxl_state->scanout_mutex);
-    free(qxl_state);
+    g_free(qxl_state);
 }
 
 Dispatcher *red_qxl_get_dispatcher(QXLInstance *qxl)
@@ -1032,20 +900,29 @@ void red_qxl_clear_pending(QXLState *qxl_state, int pending)
     clear_bit(pending, &qxl_state->pending);
 }
 
-gboolean red_qxl_get_primary_active(QXLInstance *qxl)
+bool red_qxl_get_allow_client_mouse(QXLInstance *qxl, int *x_res, int *y_res, int *allow_now)
 {
-    return qxl->st->primary_active;
-}
-
-gboolean red_qxl_get_allow_client_mouse(QXLInstance *qxl, gint *x_res, gint *y_res)
-{
-    if (qxl->st->use_hardware_cursor) {
-        if (x_res)
-            *x_res = qxl->st->x_res;
-        if (y_res)
-            *y_res = qxl->st->y_res;
+    // try to get resolution when 3D enabled, since qemu did not create QXL primary surface
+    SpiceMsgDisplayGlScanoutUnix *gl;
+    if ((gl = red_qxl_get_gl_scanout(qxl))) {
+        *x_res = gl->width;
+        *y_res = gl->height;
+        *allow_now = TRUE;
+        red_qxl_put_gl_scanout(qxl, gl);
+        return true;
     }
-    return qxl->st->use_hardware_cursor;
+
+    // check for 2D
+    if (!qxl->st->primary_active) {
+        return false;
+    }
+
+    if (qxl->st->use_hardware_cursor) {
+        *x_res = qxl->st->x_res;
+        *y_res = qxl->st->y_res;
+    }
+    *allow_now = qxl->st->use_hardware_cursor;
+    return true;
 }
 
 void red_qxl_on_ic_change(QXLInstance *qxl, SpiceImageCompression ic)
@@ -1089,6 +966,18 @@ RedsState* red_qxl_get_server(QXLState *qxl_state)
     return qxl_state->reds;
 }
 
+void red_qxl_attach_worker(QXLInstance *qxl)
+{
+    QXLInterface *interface = qxl_get_interface(qxl);
+    interface->attache_worker(qxl, &qxl->st->qxl_worker);
+}
+
+void red_qxl_set_compression_level(QXLInstance *qxl, int level)
+{
+    QXLInterface *interface = qxl_get_interface(qxl);
+    interface->set_compression_level(qxl, level);
+}
+
 void red_qxl_get_init_info(QXLInstance *qxl, QXLDevInitInfo *info)
 {
     QXLInterface *interface = qxl_get_interface(qxl);
@@ -1160,5 +1049,14 @@ void red_qxl_set_client_capabilities(QXLInstance *qxl,
 {
     QXLInterface *interface = qxl_get_interface(qxl);
 
-    interface->set_client_capabilities(qxl, client_present, caps);
+    if (qxl->st->running) {
+        interface->set_client_capabilities(qxl, client_present, caps);
+    }
+}
+
+void red_qxl_async_complete(QXLInstance *qxl, uint64_t cookie)
+{
+    QXLInterface *interface = qxl_get_interface(qxl);
+
+    interface->async_complete(qxl, cookie);
 }
