@@ -32,6 +32,8 @@
 
 #include <glib.h>
 
+#include <common/mem.h>
+
 #include "test-display-base.h"
 
 static int sized;
@@ -51,7 +53,7 @@ static void create_overlay(Command *command , int width, int height)
 
     cmd->num_clip_rects = 0;
     cmd->bitmap = g_malloc(width * height * 4 );
-    dst = (uint32_t *)cmd->bitmap;
+    dst = SPICE_ALIGNED_CAST(uint32_t *, cmd->bitmap);
     for (int i = 0; i < width * height; i++, dst++) {
         *dst = 0x8B008B;
     }
@@ -123,7 +125,7 @@ static void create_clipped_frame(Test *test, Command *command, int clipping_fact
 
     cmd->bitmap = g_malloc(width*height*4);
     memset(cmd->bitmap, 0xff, width*height*4);
-    dst = (uint32_t *)(cmd->bitmap + cur_line*width*4);
+    dst = SPICE_ALIGNED_CAST(uint32_t *, cmd->bitmap + cur_line*width*4);
     for (; cur_line < end_line; cur_line++) {
         int col;
         for (col = 0; col < width; col++, dst++) {
@@ -134,13 +136,13 @@ static void create_clipped_frame(Test *test, Command *command, int clipping_fact
         int i;
         uint32_t color = 0xffffff & rand();
 
-        dst = (uint32_t *)cmd->bitmap;
+        dst = SPICE_ALIGNED_CAST(uint32_t *, cmd->bitmap);
 
         for (i = 0; i < 50*width; i++, dst++) {
             *dst = color;
         }
 
-        dst = ((uint32_t *)(cmd->bitmap + (height - 50)*4*width));
+        dst = SPICE_ALIGNED_CAST(uint32_t *, cmd->bitmap + (height - 50)*4*width);
 
         for (i = 0; i < 50*width; i++, dst++) {
             *dst = color;
@@ -151,7 +153,7 @@ static void create_clipped_frame(Test *test, Command *command, int clipping_fact
         cmd->num_clip_rects = 0;
     } else {
         cmd->num_clip_rects = 2;
-        cmd->clip_rects = calloc(sizeof(QXLRect), 2);
+        cmd->clip_rects = g_new0(QXLRect, 2);
         cmd->clip_rects[0].left = OVERLAY_WIDTH;
         cmd->clip_rects[0].top = cmd->bbox.top;
         cmd->clip_rects[0].right = cmd->bbox.right - clipping_factor;
